package SignPredictor;

use 5.008000;
use strict;
use warnings;

require Exporter;

our @ISA = qw(Exporter);

# Items to export into callers namespace by default. Note: do not export
# names by default without a very good reason. Use EXPORT_OK instead.
# Do not simply export all your public functions/methods/constants.

# This allows declaration	use SignPredictor ':all';
# If you do not need this, moving things directly into @EXPORT or @EXPORT_OK
# will save memory.
our %EXPORT_TAGS = ( 'all' => [ qw(
	
) ] );

our @EXPORT_OK = ( @{ $EXPORT_TAGS{'all'} } );

our @EXPORT = qw(
	
);

our $VERSION = '0.01';


# File       : SignPredictor.pm
# Authors    : Vinayagam Arunachalam
# Date 	     : April 8th 2013

# Copyright (c) 2013 Vinayagam Arunachalam; Harvard Medical School

# Permission is hereby granted, free of charge, to any person
# obtaining a copy of this software and associated documentation files
# (the "Software"), to deal in the Software without restriction,
# including without limitation the rights to use, copy, modify, merge,
# publish, distribute, sublicense, and/or sell copies of the Software,
# and to permit persons to whom the Software is furnished to do so,
# subject to the following conditions:

# The above copyright notice and this permission notice shall be
# included in all copies or substantial portions of the Software.

# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
# EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
# MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
# NONINFRINGEMENT.  IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS
# BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN
# ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
# CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
# SOFTWARE.

=pod

=head1 NAME

SignPredictor - Predicts signs of protein-protein interactions
 (PPI)

=head1 SYNOPSIS

    use SignPredictor;

    my $object = SignPredictor->new(PhenotypeMatrixFile 	=>  <Phenotype matrix file name>)

    my $sign = $object->Predict_PPISign(GeneID1, GeneID2);

     GeneID1 and GeneID2 are physically interacting proteins
     
     $sign is a hash reference where 

     $sign->{PPI_sign} = predicted edge signs “+” or “-”
     $sign->{sign_score} = Sign score
     $sign->{phenotypes} = number of phenotype used to make predictions


=head1 DESCRIPTION

SignPredictor creates a object of phenotype matrix and provides functions to compute signs of PPI based on the phenotype vectors. 
The phenotype matrix is supplied as input file and edge sign for a given interacting gene pairs is obtained as output. 

=cut

use strict;

my ($zscore_cutoff, $screen_cutoff, $sign_score_cutoff);

sub Predict_PPISign {
#
# This public method returns the predicted  sign for a given gene pair
#
# Usage :
#
#    my $sign = $object->Predict_PPISign($GeneID1, $GeneID2);
#
# The method returns a hash reference
#       
#     $sign->{PPI_sign} : predicted PPI sign "+" or "-"
#
#     $sign->{sign_score} : predicted sign score 
#
#     $sign->{phenotypes} : number of phenotypes where both genes score pass the Z score cutoff value
#

	my $self = shift;
	my $gene1 = shift;
	my $gene2 = shift;
	my $sign;
	
	my ($sign_score, $phenotypes) = $self->get_gene_pair_score($gene1, $gene2);
    	if (abs($sign_score) >= $sign_score_cutoff) {
		if ($sign_score >= 1) {
			$sign->{PPI_sign} = "+";
		} else {
			$sign->{PPI_sign} = "-";
		}
		$sign->{sign_score} = $sign_score;
		$sign->{phenotypes} = $phenotypes;
	}
	return $sign;
}

sub get_gene_pair_score {
#
# This method returns a Sign score for a given gene pair
#
# Usage:
#
#    my $Sign_score = $object->get_gene_pair_score($GeneID1, $GeneID2);
#
#
  	my $self = shift;
	my $gene1 = shift;
  	my $gene2 = shift;

  	my $sign_score = 0;
  	my $phenotypes = my $positive = my $negative = 0;

  	for my $RNAiscreen ($self->get_RNAiscreenIDs) {
    		my $zscore1 = $self->get_RNAiscreen2Zscore($RNAiscreen, $gene1);
    		my $zscore2 = $self->get_RNAiscreen2Zscore($RNAiscreen, $gene2);
   
    		if (defined($zscore1) && abs($zscore1) >= $zscore_cutoff && defined($zscore2) && abs($zscore1) >= $zscore_cutoff) {
	  		$phenotypes++;
	  		if (abs($zscore1 + $zscore2) == (abs($zscore1) + abs($zscore2))) {
	     			$positive++;
	  		} else {
	    			$negative++;
	  		}
    		}
  	}
   	if ($phenotypes >= $screen_cutoff) {
		$sign_score = (($positive - $negative) /  $phenotypes) * ($phenotypes ** (1/2));
   	}
  	return ($sign_score, $phenotypes); 
}

sub get_RNAiscreen2Zscore {
#
# This method returns a Z score value for a given gene and the phenotype 
#
# Usage:
#
#    my $Zscore = $object->get_RNAiscreen2Zscore($RNAiScreenID, $geneID);
#
#    $RNAiScreenID = RNAi screen ID
#    $geneID = Gene ID
#
  	my $self = shift;
   	my $RNAiscreen = shift;
  	my $gene = shift;
  	return $self->get_value($RNAiscreen, $gene);
}

sub get_RNAiscreenIDs {
#
# This method returns an array of all the RNAi screens defined in phenotype matrix
#
# Usage:
#
#    my @RNAiScreens = $object->get_RNAiscreenIDs;
#
  	my $self = shift;
    	return $self->get_value;
}

sub get_value {
# This is private method to retrive the values from the class object

  	my $self = shift;
	my $result;
	my @results;
  	
	if (@_) {
    		my $tag1 = shift;
    		if (@_) {
      			my $tag2 = shift;
      			if (defined($self->{$tag1}->{$tag2})) {
	  			$result = $self->{$tag1}->{$tag2};
			}
			return $result;
      		} else {
			if (defined($self->{$tag1})) {
	  			@results = keys %{$self->{$tag1}};
			}
			return @results;
		}
  	} else {
    		return keys %{$self};
  	}
}

sub set_value {
# This is private method that reads the phenotype matrix

  	my $self = shift;
	my $file = shift;

	open (INP1, "<$file");
	my $l = <INP1>;
	my @head = split(/\t/, $l);
	while(<INP1>) {
		chomp;
		my @content = split(/\t/);
		my $gene = $content[0];
		for my $i (1 .. $#content) {
			my $screen = $head[$i];
			$self->{$screen}->{$gene} = $content[$i];
		}
	}
	close(INP1);
}

sub new {


# This is the constructor for SignPredictor object.
#
# The constructor expects phenotype matrix file as argument with full path to the file
#
# Usage :
#
#    my $object = SignPredictor->new(PhenotypeMatrixFile 	=>  <Phenotype matrix file name> );
#
# The phenotype matrix is supplied as input and it should be tab separated file.
#
# In the phenotype matrix file the columns correspond to phenotypes or RNAi screens (P1, P2, P3 ... Pn) and rows correspond to genes (G1, G2, G3 ... Gm). 
#
#	P1	P2	P3	P4 ... Pn
# G1
# G2
# G3
# .
# .
# .
# Gm
#
#
# The elements of the matrix, Ei,j is Z score corresponds to gene i in given RNAi screen j. 
# 
# Note: the gene ID used in the phenotype matrix should match the gene ID used in PPI network
#
# User can specify other values as follows
#
#     my $object = SignPredictor->new(PhenotypeMatrixFile 	=>  <Phenotype matrix file name>,
#					ZScoreCutoff       	=> 1.5,
#					RNAiScreenCutoff	=> 2,
#					SignScoreCutoff	=> 1);
#

	my ($class, %args) = @_;
	my $self;

	if (exists($args{'PhenotypeMatrixFile'})) {
		$self = {};
		bless $self, $class;
		
		$self->set_value($args{'PhenotypeMatrixFile'});
	}
	# Z score cutoff value
	if (exists($args{'ZScoreCutoff'})) {
		$zscore_cutoff = $args{'ZScoreCutoff'};
	} else {
		# default is 1.5
		$zscore_cutoff = 1.5;
	}
	
	# Number of RNAi screen cutoff value
	if (exists($args{'RNAiScreenCutoff'})) {
		$screen_cutoff = $args{'RNAiScreenCutoff'};
	} else {
		# default is 2
		$screen_cutoff = 2;
	}

	# Sign score cutoff value
	if (exists($args{'SignScoreCutoff'})) {
		$sign_score_cutoff = $args{'SignScoreCutoff'};
	} else {
		# default is +/- 1
		$sign_score_cutoff = 1;
	}

	return ($self);
}

1;

__END__

=pod

=head1 Instance Constructor

=head2 new

This is the constructor for SignPredictor object.

The constructor expects phenotype matrix file as argument with full path to the file

Usage :

    my $object = SignPredictor->new(PhenotypeMatrixFile 	=>  <Phenotype matrix file name> );

	The phenotype matrix is supplied as input and it should be tab separated file.

	In the phenotype matrix file the columns correspond to phenotypes or RNAi screens (P1, P2, P3 ... Pn) and rows correspond to genes (G1, G2, G3 ... Gm). 

		P1	P2	P3	P4 ... Pn
	G1
	G2
	G3
	.
	.
	.
	Gm


	The elements of the matrix, Ei,j is Z score corresponds to gene i in given RNAi screen j. 

	Note: the gene ID used in the phenotype matrix should match the gene ID used in PPI network


    To set/change Z score cutoff, number of RNAi screen cutoff and Sign score cutoff use as follows 

    my $object = SignPredictor->new(PhenotypeMatrixFile 	=>  <Phenotype matrix file name>,
					ZScoreCutoff       	=> 1.5,
					RNAiScreenCutoff	=> 2,
					SignScoreCutoff	=> 1);

ZscoreCutoff:default Z score cutoff value is 1.5, user can change it to make it stringent or relaxed
RNAiScreenCutoff: default is two or more RNAi screens, increasing the number of screen increases the precision by reduces the recall
SignScoreCutoff: default is 1, increasing the stringency improves prediction but reduces number of predictions

=head1 Instance Methods

=head2  get_RNAiscreenIDs

This method returns an array of all the RNAi screens defined in phenotype matrix

Usage:

	my @RNAiScreens = $object->get_RNAiscreenIDs;


=head2 get_RNAiscreen2Zscore

This method returns a Z score value for a given gene and the phenotype 

Usage:

	my $Zscore = $object->get_RNAiscreen2Zscore($RNAiScreenID, $geneID);

	$RNAiScreenID = RNAi screen ID
	$geneID = Gene ID


=head2 get_gene_pair_score

This method returns a Sign score for a given gene pairs

Usage:

	my $Sign_score = $object->get_gene_pair_score($GeneID1, $GeneID2);


=head2 Predict_PPISign {

This public method returns the predicted sign for a given gene pair

Usage :

	my $sign = $object->Predict_PPISign($GeneID1, $GeneID2);

	The method returns a hash reference
    
	$sign->{PPI_sign} : predicted PPI sign "+" or "-"

	$sign->{sign_score} : predicted sign score 

	$sign->{phenotypes} : number of phenotypes where both genes score pass the Z score cutoff value

=head1 Author

    Vinayagam Arunachalam; vinu@genetics.med.harvard.edu

=cut



